﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;
using Zorbo.Interface;
using System.ComponentModel;

namespace Javascript.Objects
{
    public class Avatar : ObjectInstance, IAvatar
    {
        ArrayInstance smallbytes;
        ArrayInstance largebytes;

        [JSProperty(Name = "smallBytes")]
        public ArrayInstance SmallBytes {
            get { return smallbytes; }
        }

        [JSProperty(Name = "largeBytes")]
        public ArrayInstance LargeBytes {
            get { return largebytes; }
        }

        byte[] IAvatar.SmallBytes {
            get { return smallbytes.ToArray<byte>(); }
        }

        byte[] IAvatar.LargeBytes {
            get { return largebytes.ToArray<byte>(); }
        }

        protected override string InternalClassName {
            get { return "Avatar"; }
        }

        #region " Constructor "

        public class Constructor : ClrFunction
        {
            public Constructor(ScriptEngine engine)
                : base(engine.Function.InstancePrototype, "Avatar", new Avatar(engine)) {
            }

            [JSCallFunction]
            public Avatar Call(ArrayInstance smallbytes, object largebytes) {

                if (!(largebytes is ArrayInstance))
                    return new Avatar(Engine, this.InstancePrototype, smallbytes);

                return new Avatar(Engine, this.InstancePrototype, smallbytes, (ArrayInstance)largebytes);    
            }

            [JSConstructorFunction]
            public Avatar Construct(ArrayInstance smallbytes, object largebytes) {
                return Call(smallbytes, largebytes);
            }
        }

        #endregion

        internal Avatar(ScriptEngine engine)
            : base(engine, engine.Object.InstancePrototype) {

            this.PopulateFunctions();
        }

        public Avatar(ScriptEngine engine, ObjectInstance prototype, ArrayInstance bytes)
            : base(engine, prototype) {

            this.PopulateFunctions();

            this.smallbytes = bytes;
            this.largebytes = bytes;
        }

        public Avatar(ScriptEngine engine, ObjectInstance prototype, ArrayInstance smallbytes, ArrayInstance largebytes)
            : base(engine, prototype) {

            this.PopulateFunctions();

            this.smallbytes = smallbytes;
            this.largebytes = largebytes;
        }

        public Avatar(ScriptEngine engine, IAvatar avatar)
            : base(engine) {

            this.PopulateFunctions();

            if (avatar != null) {
                this.smallbytes = avatar.SmallBytes.ToJSArray(engine);
                this.largebytes = avatar.LargeBytes.ToJSArray(engine);
            }
        }


        public bool Equals(IAvatar other) {

            if (other == null ||
                smallbytes == null ||
                other.SmallBytes == null) 
                return Object.ReferenceEquals(this, other);

            return ((IAvatar)this).SmallBytes.SequenceEqual(other.SmallBytes);
        }

        public override bool Equals(object obj) {
            return Equals(obj as Avatar);
        }


        public static bool operator ==(Avatar a, IAvatar b) {

            if (Object.ReferenceEquals(a, b))
                return true;

            if ((object)a == null || (object)b == null)
                return false;

            return a.Equals(b);
        }

        public static bool operator !=(Avatar a, IAvatar b) {
            return !(a == b);
        }

        public override int GetHashCode() {
            return base.GetHashCode();
        }
    }
}
