﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;

using Zorbo;
using Zorbo.Interface;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;

namespace Javascript.Objects
{
    public class Banned : Collection
    {
        IHistory history;

        [JSProperty(Name = "count", IsConfigurable = true)]
        public override int Count {
            get { return this.history.Bans.Count; }
        }

        public Banned(JScript script, IHistory history)
            : base(script, ((ClrFunction)script.Engine.Global["Collection"]).InstancePrototype) {

            this.script = script;
            this.history = history;

            this.PopulateFunctions();
        }

        [JSFunction(Name = "add")]
        public bool Add(object a) {

            if (a is User)
                return this.history.Bans.Add(((User)a).Client);

            else if (a is UserId)
                return this.history.Bans.Add(((UserId)a));

            else if (a is UserRecord)
                return this.history.Bans.Add(((UserRecord)a).Record);

            return false;
        }

        [JSFunction(Name = "remove")]
        public bool Remove(object a) {

            if (a is User)
                return this.history.Bans.Remove(((User)a).Client);

            else if (a is UserId) {
                return this.history.Bans.Remove((UserId)a);
            }
            else if (a is UserRecord)
                return this.history.Bans.Remove(((UserRecord)a).Record);

            else if (a is FunctionInstance) {

                var func = (FunctionInstance)a;

                for (int i = (Count - 1); i >= 0; i--) {

                    var ban = this.history.Bans[i];

                    var banned = script.Room.Records.Items.Find((s) =>
                        ((UserRecord)s).Guid == ban.Guid.ToString() ||
                        ((UserRecord)s).ExternalIp == ban.ExternalIp.ToString());

                    object ret = func.Call(Engine.Global, banned);

                    if (TypeConverter.ConvertTo<bool>(Engine, ret))
                        return this.history.Bans.RemoveAt(i);
                }
            }

            return false;
        }

        [JSFunction(Name = "removeAll")]
        public void RemoveAll(object a) {

            if (a is FunctionInstance) {

                var func = (FunctionInstance)a;

                for (int i = (Count - 1); i >= 0; i--) {

                    var ban = this.history.Bans[i];

                    var banned = script.Room.Records.Items.Find((s) =>
                        ((UserRecord)s).Guid == ban.Guid.ToString() ||
                        ((UserRecord)s).ExternalIp == ban.ExternalIp.ToString());

                    object ret = func.Call(Engine.Global, banned);

                    if (TypeConverter.ConvertTo<bool>(Engine, ret))
                        this.history.Bans.RemoveAt(i);
                }
            }
        }

        [JSFunction(Name = "removeAt")]
        public bool RemoveAt(int index) {
            return this.history.Bans.RemoveAt(index);
        }


        [JSFunction(Name = "clear")]
        public void Clear() {
            this.history.Bans.Clear();
        }



        protected override string InternalClassName {
            get { return "Banned"; }
        }

        public override IEnumerable<PropertyNameAndValue> Properties {
            get {
                for (int i = 0; i < this.Count; i++) {
                    var ban = this.history.Bans[i];

                    var banned = script.Room.Records.Items.Find((s) =>
                        ((UserRecord)s).Guid == ban.Guid.ToString() ||
                        ((UserRecord)s).ExternalIp == ban.ExternalIp.ToString());

                    yield return new PropertyNameAndValue(i.ToString(), new PropertyDescriptor(banned, PropertyAttributes.FullAccess));
                }
            }
        }

        public override PropertyDescriptor GetOwnPropertyDescriptor(uint index) {

            if (index < Count) {
                var ban = this.history.Bans[(int)index];

                var banned = script.Room.Records.Items.Find((s) =>
                        ((UserRecord)s).Guid == ban.Guid.ToString() ||
                        ((UserRecord)s).ExternalIp == ban.ExternalIp.ToString());

                return new PropertyDescriptor(banned, PropertyAttributes.FullAccess);
            }

            return new PropertyDescriptor(null, PropertyAttributes.Sealed);
        }
    }
}
