﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;

namespace Javascript.Objects
{
    public class Collection : ObjectInstance
    {
        protected JScript script;
        protected List<Object> items;

        [JSProperty(Name = "count", IsEnumerable = false, IsConfigurable = true)]
        public virtual int Count {
            get { return items.Count; }
        }

        internal List<Object> Items {
            get { return items; }
        }

        #region " Constructor "

        public class Constructor : ClrFunction
        {
            JScript script = null;

            public Constructor(JScript script)
                : base(script.Engine.Function.InstancePrototype, "Collection", new Collection(script)) {

                this.script = script;
            }

            [JSCallFunction]
            public Collection Call(object a) {
                return null;
            }

            [JSConstructorFunction]
            public Collection Construct(object a) {
                return null;
            }
        }

        #endregion

        protected override string InternalClassName {
            get { return "Collection"; }
        }


        internal Collection(JScript script)
            : base(script.Engine) {

            this.script = script;
            this.items = new List<Object>();

            this.PopulateFunctions();
        }

        public Collection(JScript script, ObjectInstance prototype)
            : base(script.Engine, prototype) {

            this.script = script;
            this.items = new List<Object>();

            this.PopulateFunctions();
        }


        [JSFunction(Name = "find")]
        public object Find(object a) {

            if (a is FunctionInstance) {
                var func = (FunctionInstance)a;

                for (int i = 0; i < items.Count; i++) {
                    object ret = func.Call(Engine.Global, items[i]);

                    if (TypeConverter.ConvertTo<bool>(Engine, ret))
                        return items[i];
                }
            }

            return null;
        }

        [JSFunction(Name = "findAll")]
        public Collection FindAll(object a) {

            if (a is FunctionInstance) {
                var func = (FunctionInstance)a;
                var list = new List(script);

                for (int i = 0; i < items.Count; i++) {
                    object ret = func.Call(Engine.Global, items[i]);

                    if (TypeConverter.ConvertTo<bool>(Engine, ret))
                        list.Add(items[i]);
                }

                return list;
            }

            return null;
        }

        [JSFunction(Name = "indexOf")]
        public int IndexOf(object a) {

            if (a is FunctionInstance) {
                var func = (FunctionInstance)a;

                for (int i = 0; i < items.Count; i++) {
                    object ret = func.Call(Engine.Global, items[i]);

                    if (TypeConverter.ConvertTo<bool>(Engine, ret))
                        return i;
                }
                return -1;
            }
            else return items.IndexOf(a);
        }

        [JSFunction(Name = "contains")]
        public bool Contains(object a) {
            return (IndexOf(a) > -1);
        }

        public override IEnumerable<PropertyNameAndValue> Properties {
            get {
                for (int i = 0; i < items.Count; i++)
                    yield return new PropertyNameAndValue(i.ToString(), new PropertyDescriptor(items[i], PropertyAttributes.FullAccess));
            }
        }

        public override PropertyDescriptor GetOwnPropertyDescriptor(uint index) {

            if (index < items.Count)
                return new PropertyDescriptor(items[(int)index], PropertyAttributes.FullAccess);

            return new PropertyDescriptor(null, PropertyAttributes.Sealed);
        }
    }
}
