﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Zorbo;
using Zorbo.Interface;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;

namespace Javascript.Objects
{
    public class Config : ObjectInstance
    {
        Avatar avatar = null;
        JScript script = null;

        IServerConfig config = null;

        protected override string InternalClassName {
            get { return "Config"; }
        }

        public Config(JScript script, IServerConfig config)
            : base(script.Engine) {

            this.config = config;
            this.script = script;

            this.PopulateFunctions();
        }

        [JSProperty(Name = "name")]
        public string Name {
            get { return config.Name; }
        }

        [JSProperty(Name = "port")]
        public int Port {
            get { return (int)config.Port; }
        }

        [JSProperty(Name = "topic")]
        public string Topic {
            get { return config.Topic; }
            set { config.Topic = value; }
        }

        [JSProperty(Name = "language")]
        public int Language {
            get { return (int)config.Language; }
            set { config.Language = (Language)value; }
        }

        [JSProperty(Name = "avatar")]
        public Avatar Avatar {
            get {
                if (avatar == null)//IAvatar might not be a JS object
                    avatar = new Avatar(Engine, config.Avatar);

                return avatar;
            }
            set {
                avatar = value;
                config.Avatar = value; 
            }
        }

        [JSProperty(Name = "botProtection")]
        public bool BotProtection {
            get { return config.BotProtection; }
            set { config.BotProtection = value; }
        }

        [JSProperty(Name = "muzzledPMs")]
        public bool MuzzledPMs {
            get { return config.MuzzledPMs; }
            set { config.MuzzledPMs = value; }
        }

        [JSProperty(Name = "maxClones")]
        public int MaxClones {
            get { return config.MaxClones; }
            set { config.MaxClones = (ushort)value; }
        }

        [JSProperty(Name = "maxClients")]
        public int MaxClients {
            get { return config.MaxClients; }
        }

        [JSProperty(Name = "expireOldPasswords")]
        public double ExpireOldPasswords {
            get { return config.ExpireOldPasswords.TotalMilliseconds; }
            set { config.ExpireOldPasswords = TimeSpan.FromMilliseconds(value); }
        }

        [JSProperty(Name = "showChannel")]
        public bool ShowChannel {
            get { return config.ShowChannel; }
            set { config.ShowChannel = value; }
        }

        [JSProperty(Name = "useChatroomBansToBanServers")]
        public bool UseChatroomBansToBanServers {
            get { return config.UseChatroomBansToBanServers; }
            set { config.UseChatroomBansToBanServers = value; }
        }
    }
}
