﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Zorbo;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;

namespace Javascript.Objects
{
    public class List : Collection
    {
        [JSProperty(Name = "count", IsConfigurable = true)]
        public override int Count {
            get { return items.Count; }
        }

        #region " Constructor "

        public new  class Constructor : ClrFunction
        {
            JScript script = null;

            public Constructor(JScript script)

                : base(script.Engine.Function.InstancePrototype, 
                       "List",
                       new List(script, ((ClrFunction)script.Engine.Global["Collection"]).InstancePrototype)) {

                this.script = script;
            }

            [JSCallFunction]
            public List Call(object a) {
                return Construct(a);
            }

            [JSConstructorFunction]
            public List Construct(object a) {
                object[] tmp = null;

                if (a is ArrayInstance) {

                    var array = (ArrayInstance)a;
                    tmp = new object[array.Length];

                    for (int i = 0; i < tmp.Length; i++)
                        tmp[i] = array[i];
                }

                if (tmp == null)
                    return new List(script, InstancePrototype);
                else
                    return new List(script, InstancePrototype, tmp);
            }
        }

        #endregion

        public List(JScript script)
            : base(script, ((ClrFunction)script.Engine.Global["Collection"]).InstancePrototype) {

            this.items = new List<Object>();
            this.PopulateFunctions();
        }

        public List(JScript script, object[] collection)
            : base(script, ((ClrFunction)script.Engine.Global["Collection"]).InstancePrototype) {

            this.items = new List<Object>();
            this.PopulateFunctions();

            collection.ForEach((s) => Add(s));
        }

        public List(JScript script, ObjectInstance prototype)
            : base(script, prototype) {

            this.items = new List<Object>();
            this.PopulateFunctions();
        }

        public List(JScript script, ObjectInstance prototype, object[] collection)
            : base(script, prototype) {

            this.items = new List<Object>();
            this.PopulateFunctions();

            collection.ForEach((s) => Add(s));
        }

        [JSFunction(Name = "add")]
        public int Add(object a) {
            if (TypeUtilities.IsPrimitiveOrObject(a))
                items.Add(a);

            return items.Count;
        }

        [JSFunction(Name = "addRange")]
        public void AddRange(object a) {
            if (a is ArrayInstance) {
                var array = (ArrayInstance)a;

                for (int i = 0; i < array.Length; i++)
                    Add(array[i]);
            }
        }

        [JSFunction(Name = "insert")]
        public int Insert(int index, object a) {
            if (TypeUtilities.IsPrimitiveOrObject(a))
                items.Insert(index, a);

            return items.Count;
        }

        [JSFunction(Name = "insertRange")]
        public void InsertRange(int index, object a) {
            if (a is ArrayInstance) {
                var array = (ArrayInstance)a;

                for (int i = 0; i < array.Length; i++)
                    Insert(index + i, array[i]);
            }
        }

        [JSFunction(Name = "remove")]
        public bool Remove(object a) {
            if (TypeUtilities.IsPrimitiveOrObject(a))
                return items.Remove(a);

            return false;
        }

        [JSFunction(Name = "removeAt")]
        public bool RemoveAt(int index) {

            if (index < 0 || index >= items.Count)
                return false;

            items.RemoveAt(index);
            return true;
        }

        [JSFunction(Name = "removeRange")]
        public void RemoveRange(int index, int count) {
            if (index < 0 || index >= items.Count)
                return;

            if (index + count > items.Count)
                count = (items.Count - index);

            items.RemoveRange(index, count);
        }

        [JSFunction(Name = "clear")]
        public void Clear() { items.Clear(); }


        protected override string InternalClassName {
            get { return "List"; }
        }

        public override void SetPropertyValue(uint index, object value, bool throwOnError) {

            if (index < items.Count)
                items[(int)index] = value;
        }

        [JSFunction(Name = "toArray")]
        public ArrayInstance ToArray() {
            var array = Engine.Array.New();

            for (int i = 0; i < Count; i++)
                array.Push(this[i]);

            return array;
        }

        public override IEnumerable<PropertyNameAndValue> Properties {
            get {
                for (int i = 0; i < items.Count; i++)
                    yield return new PropertyNameAndValue(i.ToString(), new PropertyDescriptor(items[i], PropertyAttributes.FullAccess));
            }
        }

        public override PropertyDescriptor GetOwnPropertyDescriptor(uint index) {

            if (index < items.Count)
                return new PropertyDescriptor(items[(int)index], PropertyAttributes.FullAccess);

            return new PropertyDescriptor(null, PropertyAttributes.Sealed);
        }
    }
}
