﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

using Zorbo;
using Zorbo.Interface;

using Jurassic;
using Jurassic.Library;

using JScript = Javascript.Script;

namespace Javascript.Objects
{
    public class Records : Collection
    {
        IHistory history;

        [JSProperty(Name = "count", IsConfigurable = true)]
        public override int Count {
            get { return items.Count; }
        }

        public Records(JScript script, IHistory dnsbans)
            : base(script, ((ClrFunction)script.Engine.Global["Collection"]).InstancePrototype) {

            this.history = dnsbans;
            this.history.CollectionChanged += History_Changed;

            this.items.AddRange(this.history.Select((s) => new UserRecord(script, s)));

            this.PopulateFunctions();
        }

        [JSFunction(Name = "clear", IsEnumerable = true)]
        public void Clear() {
            this.history.Clear();
        }

        private void History_Changed(object sender, NotifyCollectionChangedEventArgs e) {

            if (e.Action == NotifyCollectionChangedAction.Add)
                this.items.Add(new UserRecord(script, (IRecord)e.NewItems[0]));

            else if (e.Action == NotifyCollectionChangedAction.Remove)
                this.items.RemoveAt(e.OldStartingIndex);

            else if (e.Action == NotifyCollectionChangedAction.Reset)
                this.items.Clear();
        }

        protected override string InternalClassName {
            get { return "Records"; }
        }

        public override IEnumerable<PropertyNameAndValue> Properties {
            get {
                for (int i = 0; i < items.Count; i++)
                    yield return new PropertyNameAndValue(i.ToString(), new PropertyDescriptor(items[i], PropertyAttributes.FullAccess));
            }
        }

        public override PropertyDescriptor GetOwnPropertyDescriptor(uint index) {

            if (index < items.Count)
                return new PropertyDescriptor(items[(int)index], PropertyAttributes.FullAccess);

            return new PropertyDescriptor(null, PropertyAttributes.Sealed);
        }
    }
}
