﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Zorbo.Interface;
using Jurassic.Library;

using JScript = Javascript.Script;
using Jurassic;
using System.ComponentModel;

namespace Javascript.Objects
{
    public class User : ObjectInstance
    {
        IClient user = null;

        Avatar avatar = null;
        Avatar orgavatar = null;


        internal IClient Client {
            get { return user; }
        }

        #region " Constructor "

        public class Constructor : ClrFunction
        {
            JScript script = null;

            public Constructor(JScript script)
                : base(script.Engine.Function.InstancePrototype, "User", new User(script)) {

                this.script = script;
            }

            [JSCallFunction(Flags = JSFunctionFlags.ConvertNullReturnValueToUndefined)]
            public User Call(object a) {
                return null;
            }

            [JSConstructorFunction(Flags = JSFunctionFlags.ConvertNullReturnValueToUndefined)]
            public User Construct(object a) {
                return null;
            }
        }

        #endregion

        protected override string InternalClassName {
            get { return "User"; }
        }


        internal User(JScript script)
            : base(script.Engine) {

            this.user = new UserProto();
            this.PopulateFunctions();
        }

        public User(JScript script, IClient client)
            : base(script.Engine, ((ClrFunction)script.Engine.Global["User"]).InstancePrototype) {

            this.user = client;
            this.PopulateFunctions();
        }

        [JSProperty(Name = "id")]
        public int Id {
            get { return user.Id; }
        }

        [JSProperty(Name = "guid")]
        public string Guid {
            get { return user != null ? user.Guid.ToString() : System.Guid.Empty.ToString(); }
        }

        [JSProperty(Name = "admin")]
        public int Admin {
            get { return user != null ? (int)user.Admin : 0; }
            set { if (user != null) user.Admin = (AdminLevel)value; }
        }

        [JSProperty(Name = "loggedIn")]
        public bool LoggedIn {
            get { return user != null ? user.LoggedIn : false; }
        }

        [JSProperty(Name = "connected")]
        public bool Connected {
            get { return user != null ? user.Connected : false; }
        }

        [JSProperty(Name = "localHost")]
        public bool LocalHost {
            get { return user != null ? user.LocalHost : true; }
        }

        [JSProperty(Name = "fastPing")]
        public bool FastPing {
            get { return user.FastPing; }
        }

        [JSProperty(Name = "isCaptcha")]
        public bool IsCaptcha {
            get { return user.IsCaptcha; }
            set { user.IsCaptcha = value; }
        }

        [JSProperty(Name = "browsable")]
        public bool Browsable {
            get { return user.Browsable; }
        }

        [JSProperty(Name = "cloaked")]
        public bool Cloaked {
            get { return user.Cloaked; }
            set { user.Cloaked = value; }
        }

        [JSProperty(Name = "compression")]
        public bool Compression {
            get { return user.Compression; }
        }

        [JSProperty(Name = "muzzled")]
        public bool Muzzled {
            get { return user.Muzzled; }
            set { user.Muzzled = value; }
        }

        [JSProperty(Name = "cookie")]
        public int Cookie {
            get { return (int)user.Cookie; }
        }

        [JSProperty(Name = "age")]
        public int Age {
            get { return user.Age; }
        }

        [JSProperty(Name = "sex")]
        public int Sex {
            get { return user.Sex; }
        }

        [JSProperty(Name = "country")]
        public int Country {
            get { return user.Country; }
        }

        [JSProperty(Name = "vroom")]
        public int Vroom {
            get { return user.Vroom; }
            set { user.Vroom = (ushort)Convert.ToUInt16(value); }
        }

        [JSProperty(Name = "fileCount")]
        public int FileCount {
            get { return user.FileCount; }
        }

        [JSProperty(Name = "nodePort")]
        public int NodePort {
            get { return user.NodePort; }
        }

        [JSProperty(Name = "listenPort")]
        public int ListenPort {
            get { return user.ListenPort; }
        }

        [JSProperty(Name = "nodeIp")]
        public string NodeIp {
            get { return user.NodeIp.ToString(); }
        }

        [JSProperty(Name = "localIp")]
        public string LocalIp {
            get { return user.LocalIp.ToString(); }
        }

        [JSProperty(Name = "externalIp")]
        public string ExternalIp {
            get { return user.ExternalIp.ToString(); }
        }

        [JSProperty(Name = "dnsEntry")]
        public string DnsEntry {
            get { return (user.DnsEntry != null) ? user.DnsEntry.HostName : "error"; }
        }

        [JSProperty(Name = "avatar")]
        public Avatar Avatar {
            get {
                if (avatar == null)
                    avatar = new Avatar(Engine, user.Avatar);

                return avatar;
            }
            set {
                this.avatar = value;
                user.Avatar = value;
            }
        }

        [JSProperty(Name = "orgAvatar")]
        public Avatar OrgAvatar {
            get {
                if (orgavatar == null && user.OrgAvatar != null)
                    orgavatar = new Avatar(Engine, user.OrgAvatar);

                return orgavatar;
            }
        }
        

        [JSProperty(Name = "name")]
        public string Name {
            get { return user.Name; }
            set { user.Name = value; }
        }

        [JSProperty(Name = "orgName")]
        public string OrgName {
            get { return user.OrgName; }
        }

        [JSProperty(Name = "version")]
        public string Version {
            get { return user.Version; }
        }

        [JSProperty(Name = "region")]
        public string Region {
            get { return user.Region; }
        }

        [JSProperty(Name = "message")]
        public string Message {
            get { return user.Message; }
            set { user.Message = value; }
        }


        [JSFunction(Name = "ban", IsEnumerable = true, IsWritable = false)]
        public void Ban(object a) {
            object state = null;

            if (!(a is Undefined) && !(a is Null))
                state = a;
            
            user.Ban(state);
        }

        [JSFunction(Name = "disconnect", IsEnumerable = true, IsWritable = false)]
        public void Disconnect(object a) {
            object state = null;

            if (!(a is Undefined) && !(a is Null))
                state = a;
            
            user.Disconnect(state);
        }
    }
}
