﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Zorbo.Interface;

using Jurassic;
using Jurassic.Library;

using Javascript.Objects;

namespace Javascript
{
    public class Script
    {
        Room room;

        List<User> users;
        ScriptEngine engine;

        Dictionary<String, Int32> counters;
        static StringBuilder default_script;


        public String Name {
            get;
            private set;
        }

        internal Room Room {
            get { return room; }
        }

        internal ScriptEngine Engine {
            get { return engine; }
        }

        internal Dictionary<String, Int32> Counters {
            get { return counters; }
        }

        static Script() {
            default_script = new StringBuilder();

            default_script.AppendLine("function onError(error) { }");
            default_script.AppendLine("function onCaptcha(userobj, event) { }");
            default_script.AppendLine("function onJoinRejected(userobj, reason) { }");
            default_script.AppendLine("function onSendLogin(userobj) { }");
            default_script.AppendLine("function onJoinCheck(userobj) { return true;}");
            default_script.AppendLine("function onJoin(userobj) { }");
            default_script.AppendLine("function onPart(userobj, state) { }");
            default_script.AppendLine("function onVroomJoinCheck(userobj) { return true;}");
            default_script.AppendLine("function onVroomJoin(userobj) { }");
            default_script.AppendLine("function onVroomPart(userobj) { }");
            default_script.AppendLine("function onRegister(userobj) { return true;}");
            default_script.AppendLine("function onFileReceived(userobj, file) { return true;}");
            default_script.AppendLine("function onBeforePacket(userobj, packet) { return true;}");
            default_script.AppendLine("function onAfterPacket(userobj, packet) { }");
            default_script.AppendLine("function onFlood(userobj, packet) { return true;}");
        }

        public Script(String name) {
            this.Name = name;

            this.users = new List<User>();
            this.counters = new Dictionary<String, Int32>();

            counters.Add("print", 0);
            counters.Add("text", 0);
            counters.Add("emote", 0);
            counters.Add("private", 0);
            counters.Add("website", 0);

            this.engine = new ScriptEngine();

            engine.Global.DefineProperty("UserId",
                new PropertyDescriptor(new Objects.UserId.Constructor(this), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("Collection",
                new PropertyDescriptor(new Collection.Constructor(this), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("List",
                new PropertyDescriptor(new List.Constructor(this), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("ReadOnlyList",
                new PropertyDescriptor(new ReadOnlyList.Constructor(this), PropertyAttributes.Sealed), true);

            this.room = new Room(this, Jurassic.Self.Server);

            engine.Global.DefineProperty("User",
                new PropertyDescriptor(new User.Constructor(this), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("HttpRequest",
                new PropertyDescriptor(new HttpRequest.Constructor(this), PropertyAttributes.Sealed), true);

            engine.SetGlobalFunction("user", new Func<Object, User>((s) => room.FindUser(s)));
            engine.SetGlobalFunction("print", new Action<Object, Object>((a, b) => room.SendAnnounce(a, b)));

            engine.Global.DefineProperty("user",
                new PropertyDescriptor(Engine.Global["user"], PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("print",
                new PropertyDescriptor(Engine.Global["print"], PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("Room",
                new PropertyDescriptor(room, PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("Channels",
                new PropertyDescriptor(new Objects.Channels(this, Jurassic.Self.Server.Channels), PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("File",
                new PropertyDescriptor(new Objects.File(this), PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("Script",
                new PropertyDescriptor(new Objects.Script(this), PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("Encoding",
                new PropertyDescriptor(new Objects.Encoding.Constructor(this), PropertyAttributes.Sealed), true);

            ((Objects.Encoding)((ClrFunction)engine.Global["Encoding"]).InstancePrototype).Init();

            engine.Global.DefineProperty("Base64",
                new PropertyDescriptor(new Objects.Base64(this), PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("Hashlinks",
                new PropertyDescriptor(new Objects.Hashlinks(this), PropertyAttributes.Sealed | PropertyAttributes.Enumerable), true);

            engine.Global.DefineProperty("ChannelHash",
                new PropertyDescriptor(new Objects.ChannelHash.Constructor(this), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("Avatar",
                new PropertyDescriptor(new Objects.Avatar.Constructor(Engine), PropertyAttributes.Sealed), true);

            engine.Global.DefineProperty("FloodRule",
                new PropertyDescriptor(new Objects.FloodRule.Constructor(this), PropertyAttributes.Sealed), true);

            Eval(default_script.ToString());
        }


        public object Eval(string code) {
            return this.engine.Evaluate(code);
        }

        public T Eval<T>(string code) {
            return this.engine.Evaluate<T>(code);
        }


        public void Unload() {
            var x = Unloaded;
            if (x != null) x(this, EventArgs.Empty);

            Unloaded = null;
        }

        internal void ResetCounters() {

            var keys = counters.Select((s) => s.Key);
            foreach (var key in keys.ToArray()) counters[key] = 0;
        }

        internal event EventHandler Unloaded;
    }
}
