﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using Zorbo;//some handy extensions and things
using Zorbo.Packets;
using Zorbo.Packets.Ares;
using Zorbo.Interface;
using System.Text.RegularExpressions;

namespace Motd
{
    public class Motd : IPlugin
    {
        string mydir = "";
        string[] motdlines;

        IServer server = null;


        public object GUI {
            get { return null; }
        }

        public string Directory {
            get { return mydir; }
            set { mydir = value; }
        }


        private void LoadMotd() {
            Stream stream = null;
            StreamReader reader = null;

            try {
                string file = Path.Combine(Directory, "motd.txt");

                if (File.Exists(file)) {
                    stream = File.Open(file, FileMode.Open, FileAccess.Read);
                    reader = new StreamReader(stream);

                    server.SendAnnounce(String.Format("Motd loaded [{0} bytes]", stream.Length));
                    motdlines = reader.ReadToEnd().Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                }
            }
            catch { }
            finally {
                if (reader != null) {
                    reader.Close();
                    reader.Dispose();
                }

                if (stream != null)
                    stream.Dispose();
            }
        }


        private string ReplaceVars(IClient client, string input) {
            Regex regex = new Regex("(\\+n|\\+ip|\\+time|\\+dns|\\+vroom)");
            Match match = regex.Match(input);

            while (match.Success) {
                var group = match.Groups[1];
                string replace = string.Empty;

                switch (group.Value) {
                    case "+n":
                        replace = client.Name;
                        break;
                    case "+ip":
                        replace = client.ExternalIp.ToString();
                        break;
                    case "+time":
                        replace = TimeBank.CurrentTime.ToShortTimeString();
                        break;
                    case "+date":
                        replace = TimeBank.CurrentTime.ToShortDateString();
                        break;
                    case "+dns":
                        replace = (client.DnsEntry != null) ? client.DnsEntry.HostName : "error";
                        break;
                    case "+vroom":
                        replace = client.Vroom.ToString();
                        break;
                }

                input = input.Remove(match.Groups[1].Index, match.Groups[1].Length);
                input = input.Insert(group.Index, replace);

                match = regex.Match(input, group.Index + replace.Length);
            }

            return input;
        }


        public void OnPluginLoaded(IServer server) {
            this.server = server;
            this.server.SendAnnounce("Motd plugin has been loaded!!");

            this.LoadMotd();
        }

        public void OnPluginKilled() {
            this.server.SendAnnounce("Motd plugin has been unloaded!!");
        }

        public void OnCaptcha(IClient client, CaptchaEvent @event) {
        }

        public void OnJoinRejected(IClient client, RejectReason reason) {
        }

        public void OnSendLogin(IClient client) {
            
            //ThreadPool.QueueUserWorkItem((state) => {
            //    Thread.Sleep(500);

                if (motdlines != null)
                    motdlines.ForEach((s) =>
                        server.SendAnnounce(client, ReplaceVars(client, s)));
            //});
        }

        public bool OnJoinCheck(IClient client) {
            return true;
        }

        public void OnJoin(IClient client) {
        }

        public void OnPart(IClient client, object state) {
        }

        public bool OnVroomJoinCheck(IClient client, ushort vroom) {
            return true;
        }

        public void OnVroomJoin(IClient client) {
        }

        public void OnVroomPart(IClient client) {
        }

        public bool OnRegister(IClient client, IPassword password) {
            return true;
        }

        public bool OnFileReceived(IClient client, ISharedFile file) {
            return true;
        }

        public bool OnBeforePacket(IClient client, IPacket packet) {

            switch ((AresId)packet.Id) {
                case AresId.MSG_CHAT_CLIENT_PUBLIC:
                    ClientPublic text = (ClientPublic)packet;

                    if (text.Message.StartsWith("#"))
                        HandleCommand(client, text.Message.Substring(1));

                    break;
                case AresId.MSG_CHAT_CLIENT_COMMAND:
                    Command command = (Command)packet;
                    HandleCommand(client, command.Message);

                    break;
            }

            return true;
        }

        private void HandleCommand(IClient client, String text) {
            if (client.Admin >= AdminLevel.Admin) {
                if (text.StartsWith("loadmotd"))
                    LoadMotd();

                else if (text.StartsWith("viewmotd"))
                    if (motdlines != null)
                        motdlines.ForEach((s) =>
                            server.SendAnnounce(client, ReplaceVars(client, s)));
            }
        }

        public void OnAfterPacket(IClient client, IPacket packet) {
        }

        public bool OnFlood(IClient client, IPacket packet) {
            return true;
        }

        public void OnError(IErrorInfo error) {
        }
    }
}
