﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Forms.Integration;

using Zorbo;
using Zorbo.Interface;
using Zorbo.Plugins;

namespace Zorbo.UI
{
    /// <summary>
    /// Interaction logic for Settings.xaml
    /// </summary>
    public partial class Settings : Window
    {
        public Help Help {
            get { return (Help)base.GetValue(HelpProperty); }
        }

        public Settings(AresServer server) {

            InitializeComponent();
            DataContext = server;

            Loaded += Options_Loaded;
            Unloaded += Options_Unloaded;
        }

        private void Options_Loaded(object sender, RoutedEventArgs e) {

            AresServer server = (AresServer)DataContext;

            foreach (var plugin in ((PluginHost)server.PluginHost).Plugins) {

                TreeViewItem newitem = new TreeViewItem();

                newitem.Header = plugin.Name;
                newitem.Tag = plugin;

                FrameworkElement host = null;

                if (plugin.Interface.GUI is System.Windows.Controls.UserControl) 
                    host = (System.Windows.Controls.UserControl)plugin.Interface.GUI;
                
                else if (plugin.Interface.GUI is System.Windows.Forms.UserControl) {

                    host = new WindowsFormsHost();
                    ((WindowsFormsHost)host).Child = (System.Windows.Forms.UserControl)plugin.Interface.GUI;
                }
                else {
                    host = new TextBlock();
                    host.Height = 23;
                    host.VerticalAlignment = VerticalAlignment.Center;
                    host.HorizontalAlignment = HorizontalAlignment.Center;

                    ((TextBlock)host).Text = "The plugin did not provide a UI";
                }

                host.SetBinding(VisibilityProperty, new Binding("IsSelected") {
                    Source = newitem,
                    Converter = (BooleanToVisibilityConverter)TryFindResource("BooleanToVisibility")
                });

                Grid.SetColumn(host, 1);

                optPlugins.Items.Add(newitem);
                LayoutRoot.Children.Add(host);
            }
        }

        private void Options_Unloaded(object sender, RoutedEventArgs e) {

            foreach (var plugin in ((AresServer)DataContext).PluginHost) {

                if (plugin.GUI is System.Windows.Controls.UserControl)
                    LayoutRoot.Children.Remove((System.Windows.Controls.UserControl)plugin.GUI);

                else if (plugin.GUI is System.Windows.Forms.UserControl) {
                    foreach (var child in LayoutRoot.Children) {
                        if (child is WindowsFormsHost) {
                            var host = (WindowsFormsHost)child;
                            if (host.Child == plugin.GUI) {
                                host.Child = null;
                                LayoutRoot.Children.Remove(host);
                            }
                        }
                    }
                }
            }
        }


        public static readonly DependencyProperty HelpProperty =
            DependencyProperty.Register(
            "Help",
            typeof(Help),
            typeof(Settings),
            new PropertyMetadata(new Help()));
    }
}
